//
// Book, "RTL Modeling with SystemVerilog for ASIC and FPGA Design"
// by Stuart Sutherland and Don Mills
//
// Bitwise and arithmetic operator snippets.
//
// Copyright 2013, Sutherland HDL, Inc. All rights reserved
//
// Version 1.0
//

`begin_keywords "1800-2012"

//
// Snippet 1:
// Bitwize and arithmetic left shifts of an unsigned value
//
module shift_1;
  initial begin
  	#(1*5);    // advance to unique time slot for this test number
  	$display();
  	$display("%m: Left shift, unsigned value");
  	$display("  8'b11000101 << 2:  EXPECT 8'b00010100, ACTUAL %b", (8'b11000101 << 2));
  	$display("  8'b11000101 <<< 2: EXPECT 8'b00010100, ACTUAL %b", (8'b11000101 <<< 2));
   	$display();
  end
endmodule: shift_1


//
// Snippet 2:
// Bitwize and arithmetic right shifts of an unsigned value
//
module shift_2;
  initial begin
  	#(2*5);    // advance to unique time slot for this test number
  	$display();
  	$display("%m: Right shift, unsigned value");
  	$display("  8'b11000101 >> 2:  EXPECT 8'b00110001, ACTUAL %b", (8'b11000101 >> 2));
  	$display("  8'b11000101 >>> 2: EXPECT 8'b00110001, ACTUAL %b", (8'b11000101 >>> 2));
   	$display();
  end
endmodule: shift_2


//
// Snippet 3:
// Bitwize and arithmetic right shifts of a signed value
//
module shift_3;
  initial begin
  	#(3*5);    // advance to unique time slot for this test number
  	$display();
  	$display("%m: Right shift, signed value");
  	$display("  8'sb11000101 >> 2:  EXPECT 8'b00110001, ACTUAL %b", (8'sb11000101 >> 2));
  	$display("  8'sb11000101 >>> 2: EXPECT 8'b11110001, ACTUAL %b", (8'sb11000101 >>> 2));
   	$display();
  end
endmodule: shift_3

`end_keywords
